package com.alignet.demopayme

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.EditText
import android.widget.Spinner
import androidx.appcompat.app.AppCompatActivity
import com.alignet.payme.client.*
import com.alignet.payme.client.model.request.PAYMEAddressData
import com.alignet.payme.client.model.request.PAYMEChargesRequest
import com.alignet.payme.client.model.request.PAYMEIdentityDocument
import com.alignet.payme.client.model.request.PAYMESettingData
import com.alignet.payme.client.model.request.PAYMETransactionData
import com.alignet.payme.client.model.response.PAYMEChargesResponse
import com.alignet.payme.client.model.response.PaymeEvent
import com.google.gson.GsonBuilder
import java.util.Random

class MainActivity: AppCompatActivity(), PaymeClientDelegate {


    private lateinit var buttonInvokeSDK: Button
    private lateinit var inputEnviroment: Spinner
    private lateinit var inputMerchantCode: EditText
    private lateinit var inputOperationNumber: EditText
    private lateinit var inputEmail: EditText
    private lateinit var inputPhone: EditText
    private lateinit var inputAmount: EditText
    private lateinit var inputCurrency: EditText
    private lateinit var inputLocale: EditText
    private lateinit var inputPaymentMethods: EditText
    private lateinit var inputWalletUserCode: EditText

    private lateinit var environmentToSend: PaymeEnvironment
    private lateinit var paymeRequest: PAYMEChargesRequest
    private var events: String = ""

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.acitivty_main)

        this.buttonInvokeSDK = this.findViewById(R.id.buttonInvokeSDK)
        this.inputEnviroment = this.findViewById(R.id.inputEnviroment)
        this.inputMerchantCode = this.findViewById(R.id.inputMerchantCode)
        this.inputOperationNumber = this.findViewById(R.id.inputOperationNumber)
        this.inputEmail = this.findViewById(R.id.inputEmail)
        this.inputPhone = this.findViewById(R.id.inputPhone)
        this.inputAmount = this.findViewById(R.id.inputAmount)
        this.inputCurrency = this.findViewById(R.id.inputCurrency)
        this.inputLocale = this.findViewById(R.id.inputLocale)
        this.inputPaymentMethods = this.findViewById(R.id.inputPaymentMethods)
        this.inputWalletUserCode = this.findViewById(R.id.inputWalletUserCode)


        val actionbar = supportActionBar
        actionbar!!.title = "Testing PaymeSDK"

        this.buttonInvokeSDK.setOnClickListener { v: View -> onProcessClicked() }
        val view = (if (currentFocus == null) View(this) else currentFocus)
        val inputManager = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputManager.hideSoftInputFromWindow(view?.windowToken, InputMethodManager.SHOW_FORCED)

        val identifiers = arrayOf("DEVELOPMENT", "PRODUCTION")
        this.inputEnviroment.adapter = ArrayAdapter(this, android.R.layout.simple_spinner_dropdown_item, identifiers)
        this.inputEnviroment.onItemSelectedListener  = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(p0: AdapterView<*>?, p1: View?, p2: Int, p3: Long) {
                if (p2 == 0) {
                    this@MainActivity.environmentToSend = PaymeEnvironment.DEVELOPMENT
                }
                if (p2 == 1) {
                    this@MainActivity.environmentToSend = PaymeEnvironment.PRODUCTION
                }
            }

            override fun onNothingSelected(p0: AdapterView<*>?) {}
        }
        this.inputEnviroment.setSelection(0)
    }

    override fun onStart() {
        super.onStart()
        this.inputOperationNumber.setText((100000 + Random().nextInt(900000)).toString())
    }

    private fun onProcessClicked() {
        val customer = PAYMEAddressData(
            firstName = "Levis",
            lastName = "Silvestre",
            email = this.inputEmail.text.toString(),
            phone = this.inputPhone.text.toString(),
            identityDocument = PAYMEIdentityDocument(
                type = "DNI",
                "72661927",
                "PE"
            ),
            addressLine1 = "Av casimiro Ulloa 333",
            addressLine2 = null,
            city = "Lima",
            state = "Lima",
            country = "PE"
        )
        val billingShipping = PAYMEAddressData(
            firstName = "Levis",
            lastName = "Silvestre",
            email = this.inputEmail.text.toString(),
            phone = this.inputPhone.text.toString(),
            addressLine1 = "Av casimiro Ulloa 333",
            addressLine2 = null,
            city = "Lima",
            state = "Lima",
            country = "PE"
        )

        val reservedData = HashMap<String, String>()
        reservedData["reserved1"] = "Reservado 1"
        reservedData["reserved2"] = "Reservado 2"
        reservedData["reserved30"] = "Reservado 30"

        this.paymeRequest = PAYMEChargesRequest(
            transaction = PAYMETransactionData(
                operationNumber = this.inputOperationNumber.text.toString(),
                amount = this.inputAmount.text.toString(),
                currency = this.inputCurrency.text.toString(),
                additionalFields = reservedData,
                customer = customer,
                billing = billingShipping,
                shipping = billingShipping
            ),
            setting = PAYMESettingData(
                locale = this.inputLocale.text.toString(),
                paymentMethods = this.inputPaymentMethods.text.toString().split(","),
                walletUserID = null
            )
        )

        val paymeClient = PaymeClient(delegate = this, merchantCode = this.inputMerchantCode.text.toString())
        paymeClient.setEnvironment(environmentToSend)
        paymeClient.invokeCaptureForm(this, "SESSION_TOKEN", paymeRequest)
    }

    override fun onRespondsPayme(response: PAYMEChargesResponse) {
        val gson = GsonBuilder().setPrettyPrinting().create()
        val gsonRequest = gson.toJson(this.paymeRequest)
        val gsonResponse = gson.toJson(response)

        val intent = Intent(this, ResponseActivity::class.java)
        intent.putExtra("REQUEST_PAYME",gsonRequest)
        intent.putExtra("RESPONSE_PAYME", gsonResponse)
        intent.putExtra("EVENTS_PAYME", this.events)
        startActivity(intent)
    }

    override fun onPaymeEvents(event: PaymeEvent) {
        var eventTXT = "Event: ${event.code.name}"
        when(event) {
            is PaymeEvent.BUILD_FORM -> {
                eventTXT = "$eventTXT\nMethods: ${event.methods}"
            }
            is PaymeEvent.CHOOSE_PAYMENT_METHOD -> {
                eventTXT = "$eventTXT\nMethod: ${event.method}"
            }
            is PaymeEvent.PRESS_PAY_BUTTON -> {
                eventTXT = "$eventTXT\nMethod: ${event.method}"
            }
            is PaymeEvent.CANCEL_PAYMENT_METHOD -> {
                eventTXT = "$eventTXT\nMethod: ${event.method}"
            }
            else -> {}
        }
        this.events = "$events\n\n$eventTXT".trimStart()
    }

}